@@ -27,6 +27,8 @@ import (
 	"strings"
 	"sync"
 
+	virthandler "kubevirt.io/kubevirt/pkg/virt-handler"
+
 	"fmt"
 	"time"
 
@@ -401,6 +403,27 @@ var _ = Describe("[Serial][rfe_id:393][crit:high][vendor:cnv-qe@redhat.com][leve
 			})
 		})
 		Context("with a Cirros disk", func() {
+			It("should migrate vmi with cdroms on various bus types", func() {
+				vmi := tests.NewRandomVMIWithEphemeralDisk(cd.ContainerDiskFor(cd.ContainerDiskCirros))
+				tests.AddEphemeralCdrom(vmi, "cdrom-0", "sata", cd.ContainerDiskFor(cd.ContainerDiskCirros))
+				tests.AddEphemeralCdrom(vmi, "cdrom-1", "scsi", cd.ContainerDiskFor(cd.ContainerDiskCirros))
+				tests.AddUserData(vmi, "cloud-init", "#!/bin/bash\necho 'hello'\n")
+
+				By("Starting the VirtualMachineInstance")
+				vmi = runVMIAndExpectLaunch(vmi, 240)
+
+				By("Checking that the VirtualMachineInstance console has expected output")
+				Expect(libnet.WithIPv6(console.LoginToCirros)(vmi)).To(Succeed())
+
+				// execute a migration, wait for finalized state
+				By("starting the migration")
+				migration := tests.NewRandomMigration(vmi.Name, vmi.Namespace)
+				migrationUID := tests.RunMigrationAndExpectCompletion(virtClient, migration, migrationWaitTime)
+
+				// check VMI, confirm migration state
+				tests.ConfirmVMIPostMigration(virtClient, vmi, migrationUID)
+			})
+
 			It("[test_id:4113]should be successfully migrate with cloud-init disk with devices on the root bus", func() {
 				vmi := tests.NewRandomVMIWithEphemeralDisk(cd.ContainerDiskFor(cd.ContainerDiskCirros))
 				vmi.Annotations = map[string]string{
@@ -1475,6 +1498,94 @@ var _ = Describe("[Serial][rfe_id:393][crit:high][vendor:cnv-qe@redhat.com][leve
 				By("Deleting the VMI")
 				Expect(virtClient.VirtualMachineInstance(vmi.Namespace).Delete(vmi.Name, &metav1.DeleteOptions{})).To(Succeed())
 
+				By("Waiting for VMI to disappear")
+				tests.WaitForVirtualMachineToDisappearWithTimeout(vmi, 240)
+			})
+			It("Migration should generate empty isos of the right size on the target", func() {
+				By("Creating a VMI with cloud-init and config maps")
+				vmi := tests.NewRandomVMIWithEphemeralDisk(cd.ContainerDiskFor(cd.ContainerDiskCirros))
+				configMapName := "configmap-" + rand.String(5)
+				secretName := "secret-" + rand.String(5)
+				downwardAPIName := "downwardapi-" + rand.String(5)
+				config_data := map[string]string{
+					"config1": "value1",
+					"config2": "value2",
+				}
+				secret_data := map[string]string{
+					"user":     "admin",
+					"password": "community",
+				}
+				tests.CreateConfigMap(configMapName, config_data)
+				tests.CreateSecret(secretName, secret_data)
+				tests.AddUserData(vmi, "cloud-init", "#!/bin/bash\necho 'hello'\n")
+				tests.AddConfigMapDisk(vmi, configMapName, configMapName)
+				tests.AddSecretDisk(vmi, secretName, secretName)
+				tests.AddServiceAccountDisk(vmi, "default")
+				// In case there are no existing labels add labels to add some data to the downwardAPI disk
+				if vmi.ObjectMeta.Labels == nil {
+					vmi.ObjectMeta.Labels = map[string]string{"downwardTestLabelKey": "downwardTestLabelVal"}
+				}
+				tests.AddLabelDownwardAPIVolume(vmi, downwardAPIName)
+
+				// this annotation causes virt launcher to immediately fail a migration
+				vmi.Annotations = map[string]string{v1.FuncTestBlockLauncherPrepareMigrationTargetAnnotation: ""}
+
+				By("Starting the VirtualMachineInstance")
+				vmi = runVMIAndExpectLaunch(vmi, 240)
+
+				// execute a migration
+				By("Starting the Migration")
+				migration := tests.NewRandomMigration(vmi.Name, vmi.Namespace)
+				migration, err = virtClient.VirtualMachineInstanceMigration(migration.Namespace).Create(migration)
+				Expect(err).ToNot(HaveOccurred())
+
+				By("Waiting for Migration to reach Preparing Target Phase")
+				Eventually(func() v1.VirtualMachineInstanceMigrationPhase {
+					migration, err = virtClient.VirtualMachineInstanceMigration(migration.Namespace).Get(migration.Name, &metav1.GetOptions{})
+					Expect(err).ToNot(HaveOccurred())
+
+					phase := migration.Status.Phase
+					Expect(phase).NotTo(Equal(v1.MigrationSucceeded))
+					return phase
+				}, 120, 1*time.Second).Should(Equal(v1.MigrationPreparingTarget))
+
+				vmi, err = virtClient.VirtualMachineInstance(vmi.Namespace).Get(vmi.Name, &metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				Expect(vmi.Status.MigrationState).ToNot(BeNil())
+				Expect(vmi.Status.MigrationState.TargetPod).ToNot(Equal(""))
+
+				By("Sanity checking the volume status size and the actual virt-launcher file")
+				for _, volume := range vmi.Spec.Volumes {
+					for _, volType := range []string{"cloud-init", "configmap-", "default-", "downwardapi-", "secret-"} {
+						if strings.HasPrefix(volume.Name, volType) {
+							for _, volStatus := range vmi.Status.VolumeStatus {
+								if volStatus.Name == volume.Name {
+									Expect(volStatus.Size).To(BeNumerically(">", 0), "Size of volume %s is 0", volume.Name)
+									volPath, found := virthandler.IsoGuestVolumePath(vmi, &volume)
+									if !found {
+										continue
+									}
+									// Wait for the iso to be created
+									Eventually(func() string {
+										output, err := tests.RunCommandOnVmiTargetPod(vmi, []string{"/bin/bash", "-c", "[[ -f " + volPath + " ]] && echo found || true"})
+										Expect(err).ToNot(HaveOccurred())
+										return output
+									}, 30*time.Second, time.Second).Should(ContainSubstring("found"), volPath+" never appeared")
+									output, err := tests.RunCommandOnVmiTargetPod(vmi, []string{"/bin/bash", "-c", "/usr/bin/stat --printf=%s " + volPath})
+									Expect(err).ToNot(HaveOccurred())
+									Expect(strconv.Atoi(output)).To(Equal(int(volStatus.Size)), "ISO file for volume %s is not empty", volume.Name)
+									output, err = tests.RunCommandOnVmiTargetPod(vmi, []string{"/bin/bash", "-c", fmt.Sprintf(`/usr/bin/cmp -n %d %s /dev/zero || true`, volStatus.Size, volPath)})
+									Expect(err).ToNot(HaveOccurred())
+									Expect(output).ToNot(ContainSubstring("differ"), "ISO file for volume %s is not empty", volume.Name)
+								}
+							}
+						}
+					}
+				}
+
+				By("Deleting the VMI")
+				Expect(virtClient.VirtualMachineInstance(vmi.Namespace).Delete(vmi.Name, &metav1.DeleteOptions{})).To(Succeed())
+
 				By("Waiting for VMI to disappear")
 				tests.WaitForVirtualMachineToDisappearWithTimeout(vmi, 240)
 			})
@@ -1678,9 +1789,9 @@ var _ = Describe("[Serial][rfe_id:393][crit:high][vendor:cnv-qe@redhat.com][leve
 			}
 			tests.AddLabelDownwardAPIVolume(vmi, downwardAPIName)
 
-			vmi.Spec.Domain.Devices = v1.Devices{Interfaces: []v1.Interface{{Name: "default", Tag: "testnic",
+			vmi.Spec.Domain.Devices.Interfaces = []v1.Interface{{Name: "default", Tag: "testnic",
 				InterfaceBindingMethod: v1.InterfaceBindingMethod{
-					Masquerade: &v1.InterfaceMasquerade{}}}}}
+					Masquerade: &v1.InterfaceMasquerade{}}}}
 
 			vmi = runVMIAndExpectLaunch(vmi, 180)
 
